@@ -1,3 +1,4 @@
+import numpy as np
 from contextvars import ContextVar
 from . import ffi, replace as replace_singleton
 from .descriptor import lookup as descriptor_lookup
@@ -15,6 +16,12 @@ _recorder = ContextVar("recorder")
 _prev_recorder = None
 
 
+def record_raw(text):
+    rec = _recorder.get(_prev_recorder)
+    if rec is not None:
+        rec.record_raw(text)
+
+
 def call(cfunc_name, args):
     call_args = [getattr(x, "_carg", x) if x is not None else NULL for x in args]
     cfunc = libget(cfunc_name)
@@ -136,7 +143,7 @@ def _check_mask(mask, output=None):
 
 
 class BaseType:
-    __slots__ = "gb_obj", "dtype", "name"
+    __slots__ = "gb_obj", "dtype", "name", "__weakref__"
     # Flag for operations which depend on scalar vs vector/matrix
     _is_scalar = False
 
@@ -337,7 +344,7 @@ class BaseType:
                         scalar = delayed
                     else:
                         try:
-                            scalar = Scalar.from_value(delayed, name=repr(delayed))
+                            scalar = Scalar.from_value(delayed, name="")
                         except TypeError:
                             raise TypeError(
                                 "Assignment value must be a valid expression type, not "
@@ -406,9 +413,30 @@ class BaseType:
     _expect_type = _expect_type
     _expect_op = _expect_op
 
+    # Don't let non-scalars be coerced to numpy arrays
+    def __array__(self, dtype=None):
+        if self._is_scalar:
+            if dtype is None:
+                dtype = self.dtype.np_type
+            return np.array(self.value, dtype=dtype)
+        raise TypeError(
+            f"{type(self).__name__} can't be directly converted to a numpy array; "
+            f"perhaps use `{self.name}.to_values()` method instead."
+        )
+
 
 class BaseExpression:
-    __slots__ = "method_name", "cfunc_name", "args", "at", "bt", "op", "expr_repr", "dtype"
+    __slots__ = (
+        "method_name",
+        "cfunc_name",
+        "args",
+        "at",
+        "bt",
+        "op",
+        "expr_repr",
+        "dtype",
+        "__weakref__",
+    )
     output_type = None
 
     def __init__(
